/* apccomm_all.c
   Part of "APCComm" 
   Copyright (C) 2000 Ralf Hoffmann
   Contact: ralf.hoffmann@epost.de

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
  
*/

#include "apccomm_all.h"

unsigned char *outblock,*inblock;
pc_file_t outfile;
pc_file_t infile;

int init_transferblocks()
{
  int i;

  outblock=(unsigned char*)malloc(TRANSFERBLOCKSIZE);
  inblock=(unsigned char*)malloc(TRANSFERBLOCKSIZE);
  if(outblock!=NULL)
    for(i=0;i<1024;i++) outblock[i]=0;
  if(inblock!=NULL)
    for(i=0;i<1024;i++) inblock[i]=0;
  if((inblock==NULL)||(outblock==NULL)) return 1;
  return 0;
}

int init_outgoing_file(char*name)
{
#ifdef USE_OPEN
  int fp;
#else
  FILE *fp;
#endif
  FILE *tfp;
  int size;
  
  outfile.size=0;
  outfile.checksum=0;
#ifdef USE_OPEN
  outfile.fp=0;
#else
  outfile.fp=NULL;
#endif
  
  tfp=fopen(name,"r");
  if(tfp!=NULL) {
    fseek(tfp,0,SEEK_END);
    size=ftell(tfp);
    fseek(tfp,0,SEEK_SET);
    outfile.size=size;
#ifdef USE_OPEN
    fp=open(name,O_RDONLY);
    fclose(tfp);
#else
    fp=tfp;
#endif
    outfile.fp=fp;
  } else return 1;
  return 0;
}

int init_incoming_file(char*name)
{
  FILE *tfp;
#ifdef USE_OPEN
  int fp;
#else
  FILE *fp;
#endif
  char newname[1024],choose[16];
  int fileok;
  
  infile.size=0;
  infile.checksum=0;
#ifdef USE_OPEN
  infile.fp=0;
#else
  infile.fp=NULL;
#endif
  
  strcpy(newname,name);
  fileok=0;
  do {
    tfp=fopen(newname,"r");
    if(tfp!=NULL) {
      fclose(tfp);
      printf("File %s already exists!\nOverwrite (o) or change (c) name:",newname);
      fflush(stdout);
      scanf("%s",choose);
      if(choose[0]=='o') fileok=1;
      else {
        printf("Enter new name:");
        fflush(stdout);
        scanf("%s",newname);
      }
    } else fileok=1;
  } while(fileok==0);
#ifdef USE_OPEN
  fp=open(newname,O_WRONLY|O_CREAT|O_TRUNC,0644);
  infile.fp=fp;
  if(fp<0) return 1;
#else
  fp=fopen(newname,"w");
  infile.fp=fp;
  if(fp==NULL) return 1;
#endif
  return 0;
}

void close_outgoing_file()
{
#ifdef USE_OPEN
  if(outfile.fp!=0) close(outfile.fp);
#else
  if(outfile.fp!=NULL) fclose(outfile.fp);
#endif
}

void close_incoming_file()
{
#ifdef USE_OPEN
  if(infile.fp!=0) close(infile.fp);
#else
  if(infile.fp!=NULL) fclose(infile.fp);
#endif
}

void putint(unsigned char *buffer,int value)
{
  int b3,b2,b1,b0;
/*  int offset;
  for(offset=3;offset>=0;offset--) {
    byte=value>>(offset*8);
    byte&=0xff;
    buffer[3-offset]=(unsigned char)byte;
  }*/
  b0=value&0xff;
  value>>=8;
  b1=value&0xff;
  value>>=8;
  b2=value&0xff;
  value>>=8;
  b3=value&0xff;
  *buffer++=(unsigned char)b3;
  *buffer++=(unsigned char)b2;
  *buffer++=(unsigned char)b1;
  *buffer++=(unsigned char)b0;
}

int getint(unsigned char *buffer)
{
  int value=0;
/*  int offset;
  for(offset=0;offset<4;offset++) {
    value<<=8;
    value|=(int)buffer[offset];
  }*/
  value=(int)*buffer++;
  value<<=8;
  value|=(int)*buffer++;
  value<<=8;
  value|=(int)*buffer++;
  value<<=8;
  value|=(int)*buffer++;
  return value;
}

int readtobuffer(unsigned char* buffer,int maxsize,int *return_checksum)
{
  int readed,checksum,i;
#ifdef USE_OPEN
  readed=read(outfile.fp,buffer,maxsize);
#else
  readed=fread(buffer,1,maxsize,outfile.fp);
#endif
  checksum=0;
  for(i=0;i<readed;i++) checksum+=(int)buffer[i];
  outfile.checksum+=checksum;
  if(return_checksum!=NULL) *return_checksum=checksum;
  return readed;
}

int writefrombuffer(unsigned char *buffer,int size,int *return_checksum)
{
  int writed,checksum,i;
#ifdef USE_OPEN
  writed=write(infile.fp,buffer,size);
#else
  writed=fwrite(buffer,1,size,infile.fp);
#endif
  checksum=0;
  for(i=0;i<size;i++) checksum+=buffer[i];
  infile.checksum+=checksum;
  if(return_checksum!=NULL) *return_checksum=checksum;
  return writed;
}

