/* apccomm_pc_main.c
   Part of "APCComm" 
   Copyright (C) 2000 Ralf Hoffmann
   Contact: ralf.hoffmann@epost.de

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
  
*/

#include "apccomm_pc.h"

int main(int argc, char *argv[])
{
  struct timeval tvs,tvo,tvi;
  int secs,usecs;
  int outchecksum,inchecksum,checksum,wrong,s1,tc;
  double d1;
  int sendfile,getfile;
  int size=0,recbytes,sendbytes,ts,count;
  int sendfin,recfin;
  char outbuf[128],sendbuf[128],recvbuf[128];
  int pos=0;

  if(argc>1) {
    for(s1=1;s1<argc;s1++) {
      if((strncmp(argv[s1],"-h",2)==0)||(strncmp(argv[s1],"--help",6)==0)) {
        printf("\nAPCComm by Ralf Hoffmann\n");
        printf("  Usage: %s [<filename>]\t\tTransfer \"filename\" to the amiga\n",argv[0]);
        printf("         %s -h, --help\t\tShow this help\n",argv[0]);
        printf("         %s -v, --version\tShow program-version\n",argv[0]);
        printf("  When starting without args, APCComm will only receive a file from the Amiga\n");
        exit(0);
      } else if((strncmp(argv[s1],"-v",2)==0)||(strncmp(argv[s1],"--version",6)==0)) {
        printf("\nAPCComm by Ralf Hoffmann\n  Executable for PC-Side\n  Version %d.%d.%d\n",MAJOR,MINOR,PATCH);
        printf("\n  Programm for transfering files between Amiga and PC\n");
        printf("  Contact: ralf.hoffmann@epost.de\n");
        exit(0);
      }
    }
  }

  if(init_transferblocks()!=0) {
    fprintf(stderr,"Not enough mem in main()!\nExiting\n");
    exit(1);
  }
  
  if (pin_init_user(LPT1) < 0) {
    printf("can't initialize parallelport!\n");
    exit(0);
  }

  pin_output_mode(LP_DATA_PINS);
  printf("Waiting for Amiga:");
  fflush(stdout);
  synchro();
  printf("\nConnection established\n");
  sendfile=getfile=0;
  if(argc>1) {
    /* Datei zu uebertargen */
    if(init_outgoing_file(argv[1])!=0) {
      printf("can't open file %s!\n",argv[1]);
    } else {
      printf("sending file \"%s\", size %d\n",argv[1],outfile.size);
      sendfile=1;
    }
  }
  putint(outblock,sendfile);
  if(sendfile==1) {
    putint(outblock+4,strlen(argv[1]));
    strncpy(outblock+8,argv[1],TRANSFERBLOCKSIZE/2-12);
    putint(outblock+TRANSFERBLOCKSIZE/2-4,outfile.size);
  }
  transferblock();
  if(getint(inblock)==1) {
    if((sendfile==1)&&(strcmp(argv[1],inblock+8)==0)) {
      printf("Sending and receiving same file, Aborting\n");
      getfile=sendfile=0;
      close_outgoing_file();
    } else {
      size=getint(inblock+TRANSFERBLOCKSIZE-4);
      printf("receiving file \"%s\", size %d\n",inblock+8,size);
      init_incoming_file(inblock+8);
      getfile=1;
    }
  }
  transferblock();
  if((sendfile==0)&&(getfile==0)) {
    printf("Nothing to do!\n");
  } else {
    /* preparing progress display */
    if(sendfile!=0) {
      sprintf(outbuf,"%d",outfile.size);
      sprintf(sendbuf,"SEND: %%%dd/%%%dd (%%6.2f%%%%)",strlen(outbuf),strlen(outbuf));
      pos=6+strlen(outbuf)+1+strlen(outbuf)+2+6+1+1;
    }
    if(getfile!=0) {
      sprintf(outbuf,"%d",size);
      sprintf(recvbuf,"RECV: %%%dd/%%%dd (%%6.2f%%%%)",strlen(outbuf),strlen(outbuf));
    }
    
    printf("Starting transfer\n");
    inchecksum=outchecksum=0;
    recbytes=sendbytes=0;
    wrong=0;
    sendfin=recfin=0;
    gettimeofday(&tvs,NULL);
    for(count=0;((getfile==1)||(sendfile==1));count++) {
      if(sendfile==1) {
        ts=readtobuffer(outblock,TRANSFERBLOCKSIZE/2-4,&outchecksum);
        sendbytes+=ts;
        if(ts<1) sendfile=2;
        if(wrong==0) putint(outblock+TRANSFERBLOCKSIZE/2-4,outchecksum);
        else putint(outblock+TRANSFERBLOCKSIZE/2-4,0xffffffff);
      }
      if((sendfile==1)||(getfile==1)) transferblock();
      tc=getint(inblock+TRANSFERBLOCKSIZE-4);
      if(tc==0xffffffff) wrong=2;
      if(wrong!=0) break;
      if(getfile==1) {
        if(recbytes<size) {
          s1=size-recbytes;
          if(s1>TRANSFERBLOCKSIZE-4) s1=TRANSFERBLOCKSIZE-4;
          else getfile=2;
          writefrombuffer(inblock,s1,&inchecksum);
          recbytes+=s1;
          checksum=inchecksum-tc;
          if(checksum!=0) wrong=1;
        }
      }
      if((sendfile>1)&&(sendfin<1)) {
        gettimeofday(&tvo,NULL);
        sendfin=1;
      }
      if((getfile>1)&&(recfin<1)) {
        gettimeofday(&tvi,NULL);
        recfin=1;
      }
      if(count==19) {
        count=0;
        if((sendfile>0)&&(sendfin<2)) {
	  sprintf(outbuf,sendbuf,sendbytes,outfile.size,((double)sendbytes/outfile.size)*100);
	}
        if((getfile>0)&&(recfin<2)) {
	  if(pos>0) {
	    strcpy(outbuf+pos," - ");
	    sprintf(outbuf+pos+3,recvbuf,recbytes,size,((double)recbytes/size)*100);
	  } else sprintf(outbuf,recvbuf,recbytes,size,((double)recbytes/size)*100);
	}
	printf("\r%s",outbuf);
        fflush(stdout);
        if(sendfin==1) sendfin=2;
        if(recfin==1) recfin=2;
      }
    }
    printf("\nTransfer finished\n");
    if(wrong==1) {
      printf("Error while transfer\nI have received wrong datas!\n");
      printf("Please repeat transmission!\n");
    } else if(wrong==2) {
      printf("Error while transfer\nThe Amiga reports data loss!\n");
      printf("Please repeat transmission!\n");
    } else {
      if(sendfile>0) {
        secs=tvo.tv_sec-tvs.tv_sec;
        usecs=tvo.tv_usec-tvs.tv_usec;
        if(usecs<0) {
          secs--;
          usecs+=1000000;
        }
        d1=outfile.size;
        d1*=1000000;
        d1/=(secs*1000000.0+usecs);
        d1/=1024;
        printf("File sended with %.2g KBytes/s\n",d1);
      }
      if(getfile>0) {
        secs=tvi.tv_sec-tvs.tv_sec;
        usecs=tvi.tv_usec-tvs.tv_usec;
        if(usecs<0) {
          secs--;
          usecs+=1000000;
        }
        d1=size;
        d1*=1000000;
        d1/=(secs*1000000.0+usecs);
        d1/=1024;
        printf("File received with %.2g KBytes/s\n",d1);
      }
    }
    if(sendfile>0) close_outgoing_file();
    if(getfile>0) close_incoming_file();
  }
  free(inblock);
  free(outblock);
  return 0;
}

